# $Date: 1996/05/03 10:47:04 $ $Author: zimmerma $ $Revision: 1.2 $ #

# zimmerma #

#--
	stdlib::floatpsi -- floating point evaluation of psi


	Main reference : Abramowitz & Stegun, page 258-260 

	This implementation in exponential with respect to DIGITS,
	thus works well only for DIGITS about 10.
--#

stdlib::floatpsi := proc(z,n)
    local floatz, s, re, im, x, olds, K, DIGITS, k, X;
begin
   # Note: z must be numeric and n an non negative integer.  #
   #       if Im(z) = 0 and Re(z) <= 0 then frac(Re(z)) must #
   #       unequal zero.                                     #

   floatz:=float(z); re:=Re(floatz); im:=Im(floatz);

   #
     first use the reflection formula 6.4.7 :
     psi(1-z,n)+(-1)^(n+1)*psi(z,n)=(-1)^n*PI*diff(cot(PI*z),z$n)
   #
   if re<1/2 then
      return((-1)^n*(float(PI*subs(diff(cot(PI*x),x$n),x=1-floatz))+
             stdlib::floatpsi(1-z,n)))
   end_if;
   #
     now we have Re(z)>=1/2. use the series expansion 6.4.10 :
     psi(z,n) = (-1)^(n+1)*n!*sum((z+k)^(-n-1),k=0..infinity)
   #
   s:=0;
   K:=round(float(10^(DIGITS/(n+4))-re));
   DIGITS:=DIGITS+round(0.435*ln(1.0*K)); # ln(K)/ln(10) #
   for k from 0 to K do
       olds:=s+1/(floatz+k)^(n+1);
       if s=olds then break end_if;
          s:=olds;
   end_for;
   #
     the rest is R=sum(1/(z+k)^(n+1),k=K+1..infinity)
     which is near from I=int(1/t^(n+1),t=z+K+1/2..infinity)=1/n/(z+K+1/2)^n
     The difference in R-I=sum(a[x],x=z+K+1..infinity)
     where a[x]=1/x^(n+1)-int(1/(x+t)^(n+1),t=-1/2..1/2)
               =1/x^(n+1)*int(1-1/(1+t/x)^(n+1),t=-1/2..1/2)
               =1/x^(n+1)*[ -(n+1)*(n+2)/24/x^2 + O(1/x^4) ]
     Proof [Maple] : map(int,asympt(1-1/(1+t/x)^(n+1),x,4),t=-1/2..1/2);
     and we have
     sum(1/x^(n+3),x=X..infinity)=1/(n+2)/X^(n+2)+1/2/X^(n+3)+O(1/X^(n+4))
     Proof [Maple] : readlib(eulermac)(1/x^(n+3),x=X..infinity,2);
     Thus sum(a[x],x=X..infinity)=
              -(n+1)/24/X^(n+2)-(n+1)*(n+2)/48/X^(n+3)+O(1/X^(n+4))
     and R=1/n/(X-1/2)^n-(n+1)/24/X^(n+2)-(n+1)*(n+2)/48/X^(n+3)+O(1/X^(n+4))
     with X=z+K+1
   #
   X:=floatz+k+1;
   s:=s+1/(X-1/2)^n/n-(n+1)/24/X^(n+2)-(n+1)*(n+2)/48/X^(n+3);
   (-1)^(n+1)*fact(n)*s

end_proc:

# end of file #
